"""
You can't call grok.name multiple times for a view

  >>> import grok.tests.view.namemultiple_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: grok.name can only be called once per class.

"""
